// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package metrics

import (
	"time"

	"hello-api/internal/config"
	"hello-api/internal/metrics/wavefront"

	"github.com/opentracing/opentracing-go"
)

// Service includes metric gathering functions to be called regardless of client impl
type Service interface {
	TrackMethodTime(name string, start time.Time, dims map[string]string)

	Count(name string, dimensions map[string]string, value int64)
	Gauge(name string, dimensions map[string]string, value int64)
	Histogram(name string, dimensions map[string]string, value int64)
	RegisterMetric(name string, metric interface{}, tags map[string]string) error

	Tracer() opentracing.Tracer
	CloseConnection()
}

// NewClient returns a metrics client if configured to use one
func NewClient(cfg *config.Config) (Service, error) {
	if DoNotReport(cfg) {
		return nil, nil
	}
	return wavefront.NewReporter(cfg)
}

// DoNotReport checks to see if the client is ready to send metrics to the client used for deployment
func DoNotReport(cfg *config.Config) bool {
	return !cfg.MetricsEnabled
}
